const parsePlayer = (basePlayer, steamid, team, extensions) => {
    const extension = extensions.find(player => player.steamid === steamid);
    const player = {
        steamid,
        name: (extension && extension.name) || basePlayer.name,
        defaultName: basePlayer.name,
        clan: basePlayer.clan,
        observer_slot: basePlayer.observer_slot,
        stats: basePlayer.match_stats,
        weapons: basePlayer.weapons,
        state: { ...basePlayer.state, smoked: basePlayer.state.smoked || 0, adr: 0 },
        position: basePlayer.position.split(', ').map(pos => Number(pos)),
        forward: basePlayer.forward.split(', ').map(pos => Number(pos)),
        team,
        avatar: (extension && extension.avatar) || null,
        country: (extension && extension.country) || null,
        realName: (extension && extension.realName) || null,
        extra: (extension && extension.extra) || {}
    };
    return player;
};
export const mapSteamIDToPlayer = (players, teams, extensions) => (steamid) => parsePlayer(players[steamid], steamid, teams[players[steamid].team], extensions);
export const parseTeam = (team, orientation, side, extension) => ({
    score: team.score,
    logo: (extension && extension.logo) || null,
    consecutive_round_losses: team.consecutive_round_losses,
    timeouts_remaining: team.timeouts_remaining,
    matches_won_this_series: (extension && extension.map_score) || team.matches_won_this_series,
    side,
    name: (extension && extension.name) || team.name || (side === 'CT' ? 'Counter-Terrorists' : 'Terrorists'),
    country: (extension && extension.country) || null,
    id: (extension && extension.id) || null,
    orientation,
    extra: (extension && extension.extra) || {}
});
export const getHalfFromRound = (round, regulationMR, mr) => {
    let currentRoundHalf = 1;
    if (round <= 2 * regulationMR) {
        currentRoundHalf = round <= regulationMR ? 1 : 2;
    }
    else {
        const roundInOT = ((round - (2 * regulationMR + 1)) % (mr * 2)) + 1;
        currentRoundHalf = roundInOT <= mr ? 1 : 2;
    }
    return currentRoundHalf;
};
export const didTeamWinThatRound = (team, round, wonBy, currentRound, regulationMR, mr) => {
    // czy round i currentRound są w tej samej połowie === (czy team jest === wonBy)
    const currentRoundHalf = getHalfFromRound(currentRound, regulationMR, mr);
    const roundToCheckHalf = getHalfFromRound(round, regulationMR, mr);
    return (team.side === wonBy) === (currentRoundHalf === roundToCheckHalf);
};
export const getRoundWin = (mapRound, teams, roundWins, round, regulationMR, overtimeMR) => {
    let indexRound = round;
    if (mapRound > 2 * regulationMR) {
        const maxOvertimeRounds = 2 * overtimeMR * Math.floor((mapRound - (2 * regulationMR + 1)) / (2 * overtimeMR)) + 2 * regulationMR;
        if (round <= maxOvertimeRounds) {
            return null;
        }
        const roundInOT = ((round - (2 * regulationMR + 1)) % (overtimeMR * 2)) + 1;
        indexRound = roundInOT;
    }
    const roundOutcome = roundWins[indexRound];
    if (!roundOutcome)
        return null;
    const winSide = roundOutcome.substr(0, roundOutcome.indexOf('_')).toUpperCase();
    const result = {
        team: teams.ct,
        round,
        side: winSide,
        outcome: roundOutcome
    };
    if (didTeamWinThatRound(teams.ct, round, winSide, mapRound, regulationMR, overtimeMR)) {
        return result;
    }
    result.team = teams.t;
    return result;
};
